"use strict";

import { Fragment, useCallback, useMemo } from 'react';
import { Pressable, ScrollView, StyleSheet, Text, View, Platform } from 'react-native';
import Animated, { useAnimatedStyle, interpolate, CurvedTransition } from 'react-native-reanimated';
import { CloseIcon } from "../../components/icons/Close.js";
import { useAnimation } from "../../hooks/useAnimation.js";
import { useController } from "./controller.js";
import { styles } from "./styles.js";
import { Button } from "../Button/index.js";
import { useContainerDimensions } from "./hooks/useContainerDimensions.js";
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
export const Alert = props => {
  const {
    afterButtonsSlot,
    animationDuration,
    beforeButtonsSlot,
    beforeMessageSlot,
    beforeTitleSlot,
    renderDismissButton,
    icon,
    iconColor,
    iconSize,
    isDismissible,
    isHiding,
    testID,
    buttons,
    buttonsDirection = 'column',
    renderButton,
    title,
    renderTitle,
    message,
    renderMessage,
    config
  } = props;
  const {
    animation
  } = useAnimation({
    animationDuration,
    isHiding
  });
  const {
    onDismissButtonPress,
    onButtonPress
  } = useController(props);
  const containerDimensions = useContainerDimensions();
  const containerAnimation = useAnimatedStyle(() => {
    return {
      opacity: interpolate(animation.value, [0, 1], [0, 1]),
      transform: [{
        scaleX: interpolate(animation.value, [0, 1], [0.8, 1])
      }, {
        scaleY: interpolate(animation.value, [0, 1], [0.8, 1])
      }]
    };
  });
  const containerStyle = useMemo(() => StyleSheet.flatten([styles.container, containerDimensions, config?.alertStyle]), [containerDimensions, config?.alertStyle]);
  const beforeTitleSlotElement = useMemo(() => {
    return beforeTitleSlot?.() || config?.beforeTitleSlot?.() || null;
  }, [beforeTitleSlot, config]);
  const titleContainerStyle = useMemo(() => {
    const style = [styles.titleContainer];
    if (isDismissible && !(icon || beforeTitleSlotElement)) {
      style.push(styles.dismissibleTitleContainer);
    }
    return StyleSheet.flatten(style);
  }, [isDismissible, icon, beforeTitleSlotElement]);
  const renderTitleCb = useCallback(() => {
    const renderTitleFn = renderTitle || config?.renderTitle;
    let titleElement = null;
    if (renderTitleFn) {
      titleElement = renderTitleFn({
        style: styles.title,
        text: title ?? ''
      });
    } else if (title) {
      titleElement = /*#__PURE__*/_jsx(Text, {
        numberOfLines: 4,
        style: styles.title,
        children: title
      });
    }
    return titleElement ? /*#__PURE__*/_jsx(View, {
      style: titleContainerStyle,
      children: titleElement
    }) : null;
  }, [title, renderTitle, titleContainerStyle, config]);
  const beforeMessageSlotElement = useMemo(() => {
    return beforeMessageSlot?.() || config?.beforeMessageSlot?.() || null;
  }, [beforeMessageSlot, config]);
  const renderMessageCb = useCallback(() => {
    const renderMessageFn = renderMessage || config?.renderMessage;
    let messageElement = null;
    if (renderMessageFn) {
      messageElement = renderMessageFn({
        style: styles.message,
        text: message ?? ''
      });
    } else if (message) {
      messageElement = /*#__PURE__*/_jsx(Text, {
        style: styles.message,
        children: message
      });
    }
    return messageElement ? /*#__PURE__*/_jsx(ScrollView, {
      bounces: false,
      style: styles.messageContainer,
      children: messageElement
    }) : null;
  }, [message, renderMessage, config]);
  const renderIconCb = useCallback(() => {
    const Svg = icon;
    const iconConfig = config?.icon;
    const defaultIconSize = iconSize || iconConfig?.size || 72;
    const defaultIconColor = iconColor || iconConfig?.color || 'black';
    return Svg ? /*#__PURE__*/_jsx(View, {
      style: styles.iconContainer,
      children: /*#__PURE__*/_jsx(Svg, {
        fill: defaultIconColor,
        width: defaultIconSize,
        height: defaultIconSize
      })
    }) : null;
  }, [icon, iconColor, iconSize, config]);
  const renderDismissButtonCb = useCallback(() => {
    const defaultDismissButton = /*#__PURE__*/_jsx(Pressable, {
      onPress: onDismissButtonPress,
      style: styles.dismissButton,
      children: /*#__PURE__*/_jsx(CloseIcon, {
        width: 24,
        height: 24,
        fill: "gray"
      })
    });
    const dismissButton = isDismissible ? renderDismissButton?.({
      onPress: onDismissButtonPress
    }) || config?.renderDismissButton?.({
      onPress: onDismissButtonPress
    }) || defaultDismissButton : null;
    return dismissButton;
  }, [isDismissible, onDismissButtonPress, renderDismissButton, config]);
  const buttonsContainerStyle = useMemo(() => {
    return StyleSheet.compose(styles.buttonsContainer, {
      flexDirection: buttonsDirection,
      gap: config?.buttons?.gap || 10
    });
  }, [buttonsDirection, config]);
  const beforeButtonsSlotElement = useMemo(() => {
    return beforeButtonsSlot?.() || config?.beforeButtonsSlot?.() || null;
  }, [beforeButtonsSlot, config]);
  const afterButtonsSlotElement = useMemo(() => {
    return afterButtonsSlot?.() || config?.afterButtonsSlot?.() || null;
  }, [afterButtonsSlot, config]);
  const renderButtonsCb = useCallback(() => {
    if (buttons?.length) {
      return /*#__PURE__*/_jsx(View, {
        style: buttonsContainerStyle,
        children: buttons.map((button, i) => {
          const renderFn = renderButton || config?.buttons?.render;
          if (renderFn) {
            return /*#__PURE__*/_jsx(Fragment, {
              children: renderFn({
                text: button.text,
                onPress: () => onButtonPress(button),
                disabled: button.disabled,
                testID: button.testID,
                customProps: button.customProps
              })
            }, i);
          }
          return /*#__PURE__*/_jsx(Button, {
            text: button.text,
            onPress: () => onButtonPress(button),
            disabled: button.disabled,
            testID: button.testID
          }, i);
        })
      });
    }
    return null;
  }, [buttons, onButtonPress, buttonsContainerStyle, renderButton, config]);
  const layoutAnimation = useMemo(() => {
    return Platform.select({
      web: undefined,
      default: CurvedTransition
    });
  }, []);
  return /*#__PURE__*/_jsx(Animated.View, {
    style: containerAnimation,
    layout: layoutAnimation,
    children: /*#__PURE__*/_jsxs(View, {
      style: containerStyle,
      testID: testID || config?.testID || 'Alert',
      children: [renderIconCb(), beforeTitleSlotElement, renderTitleCb(), beforeMessageSlotElement, renderMessageCb(), beforeButtonsSlotElement, renderDismissButtonCb(), renderButtonsCb(), afterButtonsSlotElement]
    })
  });
};
//# sourceMappingURL=index.js.map